#include <lektor/common.h>
#include <lektor/reg.h>
#include <lektor/database.h>
#include <lektor/config.h>
#include <lektor/net.h>
#include <dlfcn.h>

static struct module_reg *__reg__;

void
reg_export(struct module_reg *reg)
{
    __reg__ = reg;
}

void
reg_global(struct module_reg **reg_ptr)
{
    *reg_ptr = __reg__;
}

PRIVATE_FUNCTION void *
reg_internal_get(struct module_reg *reg, const char *name)
{
    if (reg == NULL)
        return NULL;
    for (int i = 0; (reg + i)->name && (reg + i)->obj; ++i) {
        if (STR_MATCH((reg + i)->name, name))
            return (reg + i)->obj;
    }
    return NULL;
}

PRIVATE_FUNCTION reg_func
reg_internal_func(struct module_reg *reg, const char *name)
{
    if (reg == NULL)
        return NULL;
    for (int i = 0; (reg + i)->name && (reg + i)->func; ++i) {
        if (STR_MATCH((reg + i)->name, name))
            return (reg + i)->func;
    }
    return NULL;
}

PRIVATE_FUNCTION void *
reg_internal_from_file(const char *file, void **handle, const char *symbol)
{
    void *sym   = NULL;
    char *error = NULL;
    *handle     = dlopen(file, RTLD_NOW);

    if (NULL == *handle) {
        LOG_ERROR("REG", "libdl error in dlopen on file '%s': %s", file, dlerror());
        return NULL;
    }

    dlerror();
    sym   = dlsym(*handle, symbol);
    error = dlerror();

    if (error != NULL) {
        LOG_ERROR("REG", "libdl error in dlsym on file '%s': %s\n", file, error);
        return NULL;
    }

    return sym;
}

int
reg_import(const char *mod, struct module_reg **ret, void **handle)
{
    if (strcspn(mod, "/.\'\"\\") != strlen(mod)) {
        LOG_INFO("REG", "Using the dlsym to find '" REG_EXPORT_NAME "' in %s", mod);
        *ret = reg_internal_from_file(mod, handle, REG_EXPORT_NAME);
    }

    else {
        LOG_INFO("REG", "Using the reg structure to find %s", mod);
        *ret = reg_internal_get(__reg__, mod);
        if (!*ret) {
            LOG_FATAL("The module %s was not found by using the reg '" REG_EXPORT_NAME "' or by "
                      "using dlsym to find reg '" REG_EXPORT_NAME "' in file %s",
                      mod);
        }
    }
    return *ret != NULL;
}

int
reg_call(struct module_reg *reg, const char *name, int count, ...)
{
    reg_func func = reg_internal_func(reg, name);
    if (!func) {
        LOG_ERROR("REG", "No function named '%s' in reg: %s", name, reg->name);
        return 2;
    }
    va_list va;
    va_start(va, count);
    int ret = func(&va);
    va_end(va);
    return ret;
}
