use crate::constants;
use amadeus_macro::either;

pub fn toggle_switch(dark_mode: bool, on: &mut bool) -> impl egui::Widget + '_ {
    move |ui: &mut egui::Ui| {
        let desired_size = ui.spacing().interact_size.y * egui::vec2(2.0, 1.0);
        let (rect, mut response) = ui.allocate_exact_size(desired_size, egui::Sense::click());
        if response.clicked() {
            *on = !*on;
            response.mark_changed();
        }
        response.widget_info(|| egui::WidgetInfo::selected(egui::WidgetType::Checkbox, *on, ""));

        if ui.is_rect_visible(rect) {
            let (bg_fill, bg_stroke, fg_stroke, expansion) = {
                let visuals = ui.style().interact_selectable(&response, *on);
                (
                    visuals.bg_fill,
                    visuals.bg_stroke,
                    visuals.fg_stroke,
                    visuals.expansion,
                )
            };
            let fill_fore = constants::get_accent_color(dark_mode);
            let fill_back = constants::get_fill_color(dark_mode);
            let rect = rect.expand(expansion);
            let radius = 0.5 * rect.height();

            ui.painter()
                .rect(rect, radius, either!(*on => fill_back; bg_fill), bg_stroke);

            let circle_x = egui::lerp(
                (rect.left() + radius)..=(rect.right() - radius),
                ui.ctx().animate_bool(response.id, *on),
            );
            let center = egui::pos2(circle_x, rect.center().y);

            ui.painter()
                .circle(center, 0.75 * radius, fill_fore, fg_stroke);
        }

        response
    }
}
